#include <qlayout.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qpe/qcopenvelope_qws.h>
#include <qmessagebox.h>
#include <qstringlist.h>
#include <qpe/config.h>
#include <qlistbox.h>
#include <qdatetime.h>

#include "SendQ.h"


SendQ::SendQ(QWidget * parent, const char * name , WFlags f) : QWidget(parent,name ,f)
{
	layout = new QGridLayout(this);
	layout->setSpacing( 6 );
	layout->setMargin( 11 );
	layout->setColStretch(Tail,1);
	layout->setColStretch(Top,1);

	Config cf = Config("qcoptest");

	chNameLabel = new QLabel(this);
	chNameLabel->setText(tr("Channel Name"));
	layout->addWidget(chNameLabel,Channel,Top);
	chPreLabel = new QLabel(this);
	chPreLabel->setText("QPE/");
	chPreLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	layout->addWidget(chPreLabel,Channel,Mid);
	chNameBox = new QComboBox(true,this);
	chNameBox->setAutoCompletion(true);
	chNameBox->setDuplicatesEnabled(false);
	layout->addWidget(chNameBox,Channel,Tail);
	cf.setGroup("ChHistory");
	addItem(chNameBox,"System");
	addItem(chNameBox,"TaskBar");
	addItem(chNameBox,"Network");
	addItem(chNameBox,"Card");
	addItem(chNameBox,"Sync");
	QString h = "";
	for ( int i = 0 ; !(h = cf.readEntry(QString("hist%1").arg(i)) ).isEmpty() ; ++ i) {
		addItem(chNameBox,h);
	}

	comNameLabel = new QLabel(this);
	comNameLabel->setText(tr("Command"));
	layout->addWidget(comNameLabel,Command,Top);
	comNameBox = new QComboBox(true,this);
	comNameBox->setAutoCompletion(true);
	comNameBox->setDuplicatesEnabled(false);
	layout->addMultiCellWidget(comNameBox,Command,Command,Mid,Tail);
	cf.setGroup("ComHistory");
	h = "";
	for ( int i = 0 ; !(h = cf.readEntry(QString("hist%1").arg(i)) ).isEmpty() ; ++ i) {
		addItem(comNameBox,h);
	}

	for (int i = 0 ; i < ArgNum ; ++i) {
		argLabel[i] = new QLabel(this);
		argLabel[i]->setText(tr("Argument %1").arg(i+1));
		layout->addWidget(argLabel[i],Arg1+i,Top);
		argBox[i] = new QComboBox(true,this);
		argBox[i]->setMaximumWidth(sizeHint().width());
		argBox[i]->setAutoCompletion(true);
		argBox[i]->setDuplicatesEnabled(false);
		layout->addMultiCellWidget(argBox[i],Arg1+i,Arg1+i,Mid,Tail);
		cf.setGroup(QString("Arg%1History").arg(i+1));
		h = "";
		for ( int j = 0 ; !(h = cf.readEntry(QString("hist%1").arg(j)) ).isEmpty() ; ++j) {
			addItem(argBox[i],h);
		}
	}

	sendButton = new QPushButton(this);
	sendButton->setMaximumWidth(sizeHint().width());
	sendButton->setText(tr("Send QCop"));
	layout->addWidget(sendButton,Button,Mid);

	connect(sendButton,SIGNAL(clicked()) , this , SLOT(pushSendButton()) );
}

SendQ::~SendQ()
{
	Config cf = Config("qcoptest");
	cf.setGroup("ChHistory");
	cf.clearGroup();
	for (int i = 0 ; i < chNameBox->count() ; ++i) {
		cf.writeEntry(QString("hist%1").arg(i),chNameBox->text(i));
	}
	cf.setGroup("ComHistory");
	cf.clearGroup();
	for (int i = 0 ; i < comNameBox->count() ; ++i) {
		cf.writeEntry(QString("hist%1").arg(i),comNameBox->text(i));
	}

	for (int j = 0 ; j < ArgNum ; ++j) {
		cf.setGroup(QString("Arg%1History").arg(j+1));
		cf.clearGroup();
		for (int i = 0 ; i < argBox[j]->count() ; ++i) {
			cf.writeEntry(QString("hist%1").arg(i),argBox[j]->text(i));
		}
	}
}

void SendQ::pushSendButton()
{
	QString ch = chNameBox->currentText();
	QCString cch = "QPE/"+ QCString(ch);
	QString com = comNameBox->currentText();
	QCString ccom = QCString(com);
	
	int s = com.find('(');
	int t = com.find(')');
	if (s < 0 || t < 0) {
		QMessageBox::warning(this, tr("Error"),tr("Command format wrong. '(' and ')' expected.")) ;
	} else {
		addItem(chNameBox,ch);
		addItem(comNameBox,com);
		for ( int i = 0 ; i < ArgNum ; ++i) {
			addItem(argBox[i],argBox[i]->currentText());
		}
		QString a = (com.left(t)).mid(s+1);
		QStringList arg = QStringList::split(',',a);
		QCopEnvelope e(cch, ccom);
		int i = 0;
		for (QStringList::Iterator it = arg.begin()  ; it != arg.end() ; ++it , ++i) {
			QString args = argBox[i]->currentText();
			if (*it == "QString") {
				e << args;
			} else if (*it == "int") {
				e << args.toInt();
			} else if (*it == "bool") {
				if (args.lower() == "true") {
					e << 1;
				} else if (args.lower() == "false") {
					e << 0;
				} else {
					e << args.toInt();
				}
			} else if (*it == "QCString") {
				e << QCString(args);
			} else if (*it == "QStringList") {
				e << QStringList::split(' ',args,true);
			} else if (*it == "QDateTime") {
				QStringList dt = QStringList::split(' ',args,true);
				QStringList dd = QStringList::split('/',dt[0],true);
				QStringList tt = QStringList::split(':',dt[1],true);
				if (dd.count() != 3 || tt.count() != 3 ) {
					QMessageBox::warning(this, tr("Error"),tr("DateTime format wrong.")) ;
					break;
				}
				QDate da(dd[0].toInt(),dd[1].toInt(),dd[2].toInt());
				QTime ti(tt[0].toInt(),tt[1].toInt(),tt[2].toInt());
				QDateTime dat(da,ti);
				e << dat;
			} else {
				QMessageBox::warning(this, tr("Error"),tr("Command format wrong. not supported type.")) ;
				break;
			}
		}
	}
}

void SendQ::addItem(QComboBox* b,const QString& s)
{
	if (b->listBox()->findItem(s) == NULL && ! s.isEmpty()) {
		b->insertItem(s);
	}
}
