#include <qpe/qpeapplication.h>
#include <qdatastream.h>
#include <qstring.h>
#include <qcstring.h>
#include <qdatetime.h>
#include <qpe/mimetype.h>
#include <qpe/config.h>

#include "qcoptest.h"

QcopTest::QcopTest() : QMainWindow()
{
	setCaption(tr("qcoptest")) ;
	setToolBarsMovable(false) ;
	tabWidget = new QTabWidget( this);

	sendQ = new SendQ(tabWidget);
	tabWidget->insertTab(sendQ,"Send QCop");

	sysView = new View(tabWidget);
	tabWidget->insertTab(sysView,"System");
	taskView = new View(tabWidget);
	tabWidget->insertTab(taskView,"TaskBar");
	netView = new View(tabWidget);
	tabWidget->insertTab(netView,"Network");
	cardView = new View(tabWidget);
	tabWidget->insertTab(cardView,"Card");
	syncView = new View(tabWidget);
	tabWidget->insertTab(syncView,"Sync");
	userView = NULL;

	Config cf = Config("qcoptest");
	cf.setGroup("config");
	userChName = cf.readEntry("ChannelName");

	pre = cf.readEntry("LogDirectory",QPEApplication::documentDir());

	cf.setGroup("ignore");
	for ( int i = 0 ; i < 100 ; ++i ) {
		QString il = cf.readEntry(QString("entry%1").arg(i)) ;
		if (! il.isEmpty() ) {
			ignore.append(il);
		}
	}

	sysFile.setName(pre + "system.txt");
	sysFile.open(IO_WriteOnly | IO_Append);
	sysSt.setDevice(&sysFile);
	taskFile.setName(pre + "taskbar.txt");
	taskFile.open(IO_WriteOnly | IO_Append);
	taskSt.setDevice(&taskFile);
	netFile.setName(pre + "network.txt");
	netFile.open(IO_WriteOnly | IO_Append);
	netSt.setDevice(&netFile);
	cardFile.setName(pre + "card.txt");
	cardFile.open(IO_WriteOnly | IO_Append);
	cardSt.setDevice(&cardFile);
	syncFile.setName(pre + "sync.txt");
	syncFile.open(IO_WriteOnly | IO_Append);
	syncSt.setDevice(&syncFile);

	QString d = "Logging Start at " + QDateTime::currentDateTime().toString() + "\n";
	sysSt << d ;
	taskSt << d;
	netSt << d;
	cardSt << d;
	syncSt << d;

	setCentralWidget(tabWidget);

	QCopChannel *channel = new QCopChannel("QPE/System", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(systemMessage(const QCString&, const QByteArray&)));

	channel = new QCopChannel("QPE/TaskBar", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(taskBarMessage(const QCString&, const QByteArray&)));
	channel = new QCopChannel("QPE/Network", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(networkMessage(const QCString&, const QByteArray&)));
	channel = new QCopChannel("QPE/Card", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(cardMessage(const QCString&, const QByteArray&)));
	channel = new QCopChannel("QPE/Sync", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(syncMessage(const QCString&, const QByteArray&)));

	setUserChannel();
}

QcopTest::~QcopTest() 
{
	sysFile.close();
	taskFile.close();
	netFile.close();
	cardFile.close();
	syncFile.close();
	userFile.close();
	Config cf = Config("qcoptest");
	cf.setGroup("config");
	cf.writeEntry("ChannelName",userChName);
	cf.writeEntry("LogDirectory",pre);

}

void QcopTest::setDocument(const QString &s)
{
	userChName = s;
	setUserChannel();
}

void QcopTest::systemMessage(const QCString& msg, const QByteArray& data)
{
	processMessage(sysView,&sysSt,"System" ,msg,data);
}

void QcopTest::taskBarMessage(const QCString& msg, const QByteArray& data)
{
	processMessage(taskView,&taskSt,"TaskBar" ,msg,data);
}
void QcopTest::networkMessage(const QCString& msg, const QByteArray& data)
{
	processMessage(netView,&netSt,"Network" ,msg,data);
}
void QcopTest::cardMessage(const QCString& msg, const QByteArray& data)
{
	processMessage(cardView,&cardSt,"Card",msg, data);
}
void QcopTest::syncMessage(const QCString& msg, const QByteArray& data)
{
	processMessage(syncView,&syncSt,"Sync",msg, data);
}
void QcopTest::userMessage(const QCString& msg, const QByteArray& data)
{
	processMessage(userView,&userSt,userChName, msg,data);
}

void QcopTest::processMessage(View* view, QTextStream* st ,const QString& chName , const QCString& msg, const QByteArray& data)
{
	if (ignore.findIndex(chName + " " + QString(msg)) >= 0 ) {
		return;
	}
	QDataStream stream(data, IO_ReadOnly);
	QString r = QTime::currentTime().toString() + " - Recv - " + QString(msg) ;
	view->insertLine(r);
	*st << r << "\n";

	QString a = QString(msg);
	if (a == "goingToSendSetDocument(QString)") {
		return;
	}
	a = a.mid(a.find('(')+1);
	a = a.left(a.find(')'));
	QStringList arg = QStringList::split(',',a);
	for (QStringList::Iterator it = arg.begin() ; it != arg.end() ; ++it) {
		QString o = QString("%1 : ").arg(*it,13);
		if (*it == "QString") {
			QString s ;
			stream >> s;
			o += "\"" + s + "\"";
		} else if (*it == "int") {
			int i;
			stream >> i;
			o += QString("%1").arg(i);
		} else if (*it == "bool") {
			int i;
			stream >> i;
			o += i ? "TRUE" : "FALSE";
		} else if (*it == "QCString") {
			QCString s ;
			stream >> s;
			o += "\"" + QString(s) + "\"";
		} else if (*it == "QStringList") {
			QStringList s ;
			stream >> s;
			o += "\"" + s.join("\" \"") + "\"";
		} else if (*it == "QDateTime") {
			QDateTime d ;
			stream >> d;
			o += d.toString();
		}
		view->insertLine(o);
		*st << o << "\n";
	}
	view->setCursorPosition(view->numLines(),0);
	st->device()->flush();
}

void QcopTest::setUserChannel()
{
	if (userChName.isEmpty()) {
		return;
	}
	if (! QCopChannel::isRegistered(QCString("QPE/"+ userChName)) ) {
		if (userView ) {
			userView->insertLine("Channel " + userChName + " is not resident.");
		}
		return;
	}
	
	if (userView) {
		userFile.close();
		QString old = QString(userCh->channel());
		disconnect(userCh, SIGNAL(received(const QCString&, const QByteArray&)),
			this, SLOT(userMessage(const QCString&, const QByteArray&)));
		delete userCh;
		tabWidget->changeTab(userView,userChName);
		userView->insertLine("user ch changed from "+old + " to " +"QPE/" + userChName);
	} else {
		userView = new View(tabWidget);
		tabWidget->insertTab(userView,userChName);
	}

	userFile.setName(pre + "userchannel.txt");
	userFile.open(IO_WriteOnly | IO_Append);
	userSt.setDevice(&userFile);

	QString d = "channel QPE/" + userChName + "\nLogging Start at " + QDateTime::currentDateTime().toString() + "\n";
	userSt << d;
	QCopChannel *channel = new QCopChannel(QCString("QPE/"+ userChName), this);
	userCh = channel;
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(userMessage(const QCString&, const QByteArray&)));
}
